@@ -1,4 +1,4 @@
-from django.conf.urls import url
+from django.urls import re_path as url
 from . import views
 
 app_name = 'diary'
